"""
$RCSfile: ITransformerRegistry.py,v $

XMLTransform automatically transforms an XML document via XSLT, where
the XML document is obtained from another Zope object via acquisition.
ITransformerRegistry is the Interface implemented by
TransformerRegistry.

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/12 20:52:44 $'[6:-2]
__version__ = '$Revision: 1.8 $'[10:-2]

# Zope Base Classes
from Interface import Base

################################################################
# ITransformerRegistry interface
################################################################

class ITransformerRegistry(Base):
    """

    Interface for a registry of XSLT transformer stylesheets to be
    used with instances of XML Transform.  The default implementation
    will not enforce any particular policy nor support any special
    semantics (for example, "knowing" that a particular transformer
    has both HTML and FO flavors), but subclasses are free to do so.
    See comments in TransformerRegistry for more details.

    """

    def transformerNames(self):
        """

        Returns the names of transformers registered with this
        registry.  Different implementations will have different
        policies for how the names are constructed and returned, but
        they must be such that, if passed back to transformerObject()
        verbatim, they will yield the proper object.

        """

    def transformerMetaTypes(self):
        """

        Returns the types of objects that are allowed to be
        transformers.  Objects contained within the registry that have
        meta_types that are not in this list will be omitted, that is,
        their names will not be returned via transformerNames()

        """

    def transformerObject(self, transformerName):
        """
        
        Given the name (as retrieved using the transformerNames()
        method), return the actual zope object corresponding to the
        XSLT transformer stylesheet.  Return None if the name does not
        point to a valid object.
        
        """
