"""
$RCSfile: TestXMLTransform.py,v $
Unit tests for XMLTransform product

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/26 19:00:00 $'[6:-2]
__version__ = '$Revision: 1.12 $'[10:-2]

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
from Testing import ZopeTestCase

# XMLTransform
from Products.XMLTransform import CacheManager 
from Products.XMLTransform import TransformerRegistry
from Products.XMLTransform import XMLTransform

ZopeTestCase.installProduct('XMLTransform')

class XMLTransformTestCase(ZopeTestCase.ZopeTestCase):
    """

    Unit tests for XMLTransform Zope Product.  These tests use the
    ZopeTestCase enhanced testing product.

    """

    ################################################################
    # Fixture
    ################################################################

    def afterSetUp(self):
        '''Add object to default fixture'''
        pass

    def afterClear(self):
        '''Clean up after myself'''
        pass
    ################################################################
    # Test Cases
    ################################################################

    def test_01(self):
        "Ensure that XMLTransform cannot be created with a blank ID"
        self.assertRaises(Exception, XMLTransform.addInstance,
                          self.folder,id='',title='',description='',
                          transformerName='',sourceID='')

    def test_02(self):
        "Ensure that XMLTransform cannot be created without a Registry"
        self.assertRaises(Exception,XMLTransform.addInstance,
                          self.folder,id='aXMLTransform',title='',description='',
                          transformerName='aTransformer',sourceID='aSource')

    def test_03(self):
        "Ensure that XMLTransform cannot be created without a valid transformer"
        reg = TransformerRegistry.addInstance(self.folder,'registry')        
        self.assertRaises(Exception, XMLTransform.addInstance,
                          self.folder,id='aXMLTransform',title='',description='',
                          transformerName='aTransformer',sourceID='aSource')

    def test_04(self):
        "Ensure that XMLTransform cannot be created without a source"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        reg.manage_addDTMLMethod('aTransformer')
            
        self.assertRaises(Exception, XMLTransform.addInstance,
                          self.folder,id='aXMLTransform',title='',description='',
                          transformerName='aTransformer',sourceID='aSource')

    def test_05(self):
        "Ensure that XMLTransform can be created with minimal correct params"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        reg.manage_addDTMLMethod('aTransformer')
        self.folder.manage_addDTMLMethod('aSource')
        src = self.folder['aSource']

        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        self.failUnless(xform, "XMLTransform successfully created should not be None")
        self.assertEquals(xform.id, 'aXMLTransform')
        self.failIf(xform.findCacheManager(),"findCacheManager should return None")
        self.assertEquals(reg, xform.findRegistry(), "findRegistry should return reg")
        self.assertEquals(src, xform.findSourceObject(),"findSourceObject should return src")
        self.assertEquals(1, xform.isCachingOn(), "Caching should be on by default")

        xform.setCachingOff()
        self.assertEquals(0, xform.isCachingOn(), "Caching should be turned off")

    def test_06(self):
        "Ensure that XMLTransform can be created if there are two registries available"
        reg1 = TransformerRegistry.addInstance(self.folder,'registry1')
        reg1.manage_addDTMLMethod('aTransformer')

        reg2 = TransformerRegistry.addInstance(self.folder,'registry2')
        reg2.manage_addDTMLMethod('aTransformer')
        
        self.folder.manage_addDTMLMethod('aSource')

        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        self.failUnless(xform, "XMLTransform successfully created should not be None")
        self.assertEquals(xform.id, 'aXMLTransform')

    def test_07(self):
        "Ensure that basic transform works correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        
        self.assertEquals(xform.transform(self.app.REQUEST), "Hello, world")

    def test_08(self):
        "Ensure that transformation obtains scalar parameters correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/param.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.XSLparameters = [ "who", "message" ]
        self.folder.who           = "sneezy"
        self.folder.message       = "take some claritin"
        
        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        result = xform.transform(self.app.REQUEST)
        self.assertEquals(result, "Hello, sneezy and take some claritin")

    def test_09(self):
        "Ensure that transformation obtains zope object parameters correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/propertiesfile.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.XSLparameters = [ "properties" ]
        
        propFile      = open('testfiles/properties.xml', 'rb')
        propContents  = propFile.read()
        self.folder.manage_addDTMLMethod('properties','',propContents)
        
        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        self.assertEquals(xform.transform(self.app.REQUEST),
                         "url is http://foo/testFolder_1_/properties")

#      def test_10(self):
#          """Ensure that transformation resolves URLs in documents correctly
#          We don't do this yet"""
#          reg = TransformerRegistry.addInstance(self.folder,'registry')

#          xsltFile     = open('testfiles/paramfile.xsl','rb')
#          xsltContents = xsltFile.read()
#          reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
#          xmlFile      = open('testfiles/param.xml', 'rb')
#          xmlContents  = xmlFile.read()
#          self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

#          self.folder.URNnamespaces = [ "arielpartners" ]
#          self.folder.XSLparameters = [ "properties" ]
#          self.folder.properties    = "http://foo/testFolder_1_/arielpartners/propfolder/foo/properties"
#          #self.folder.properties    = "../arielpartners/propfolder/foo/properties"

#          self.folder.manage_addFolder('arielpartners')
#          arielpartners = self.folder['arielpartners']
#          arielpartners.manage_addFolder('propfolder')
#          propfolder = arielpartners['propfolder']
#          propfolder.manage_addFolder('foo')
#          foo = propfolder['foo']
        
#          propFile      = open('testfiles/properties.xml', 'rb')
#          propContents  = propFile.read()
#          foo.manage_addDTMLMethod('properties','',propContents)
        
#          xform = XMLTransform.addInstance(arielpartners,id='aXMLTransform',title='',description='',
#                                           transformerName='aTransformer',sourceID='aSource')

#          self.assertEquals(xform.transform(self.app.REQUEST), "Hello, sneezy")

    def test_11(self):
        "Ensure that transformation resolves URNs in documents correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/calldocumentfunc.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.URNnamespaces = [ "arielpartners" ]
        
        self.folder.manage_addFolder('arielpartners')
        arielpartners = self.folder['arielpartners']
        arielpartners.manage_addFolder('propfolder')
        propfolder = arielpartners['propfolder']
        propfolder.manage_addFolder('foo')
        foo = propfolder['foo']
        
        propFile      = open('testfiles/properties.xml', 'rb')
        propContents  = propFile.read()
        foo.manage_addDTMLMethod('properties','',propContents)
        
        xform = XMLTransform.addInstance(arielpartners,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        self.assertEquals(xform.transform(self.app.REQUEST), "Hello, sneezy")

    def test_12(self):
        "Ensure that transformation resolves URNs in transformers correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/include.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.URNnamespaces = [ "arielpartners" ]

        self.folder.manage_addFolder('arielpartners')
        arielpartners = self.folder['arielpartners']
        arielpartners.manage_addFolder('xforms')
        xforms = arielpartners['xforms']
        
        xsltFile      = open('testfiles/simple.xsl', 'rb')
        xsltContents  = xsltFile.read()
        xforms.manage_addDTMLMethod('simple','',xsltContents)
        
        xform = XMLTransform.addInstance(arielpartners,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        self.assertEquals(xform.transform(self.app.REQUEST), "Hello, world")

    def test_13(self):
        "Verify error report if stylesheet URN resolver cannot find included stylesheet"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/include.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.URNnamespaces = [ "arielpartners" ]

        self.folder.manage_addFolder('arielpartners')
        arielpartners = self.folder['arielpartners']
        arielpartners.manage_addFolder('xforms')
        xforms = arielpartners['xforms']

        # Nope, not there.  
        #xsltFile      = open('testfiles/simple.xsl', 'rb')
        #xsltContents  = xsltFile.read()
        #xforms.manage_addDTMLMethod('simple','',xsltContents)
        
        xform = XMLTransform.addInstance(arielpartners,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        try:
            xform.testTransform(self.app.REQUEST)
        except Exception, e:
            self.failIf(str(e).find("urn:arielpartners:xforms/simple") == -1)

    def test_14(self):
        "Verify error report if document URN resolver cannot find included document"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/calldocumentfunc.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.URNnamespaces = [ "arielpartners" ]

        self.folder.manage_addFolder('arielpartners')
        arielpartners = self.folder['arielpartners']
        arielpartners.manage_addFolder('propfolder')
        propfolder = arielpartners['propfolder']

        # Nope.  not there.
        #propFile      = open('testfiles/properties.xml', 'rb')
        #propContents  = propFile.read()
        #propfolder.manage_addDTMLMethod('properties','',propContents)
        
        xform = XMLTransform.addInstance(arielpartners,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        try:
            xform.testTransform(self.app.REQUEST)
        except Exception, e:
            self.failIf(str(e).find("document") == -1)

    def test_15(self):
        "Verify error report if document URN resolver gets badly formed URN"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/paramfile.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.URNnamespaces = [ "arielpartners" ]
        self.folder.XSLparameters = [ "properties" ]
        self.folder.properties    = "urn:arielpartners:propfolder/i/dont/exist"

        self.folder.manage_addFolder('arielpartners')
        arielpartners = self.folder['arielpartners']
        arielpartners.manage_addFolder('propfolder')
        propfolder = arielpartners['propfolder']
        propfolder.manage_addFolder('i')
        i = propfolder['i']
        i.manage_addFolder('dont')
        dont = i['dont']
        dont.manage_addFolder('exist')

        xform = XMLTransform.addInstance(arielpartners,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')

        self.assertRaises(Exception, xform.testTransform, self.app.REQUEST)
        
    def test_16(self):
        "Ensure that transformation resolves DTML correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/dtmltest.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.who = "sneezy"

        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, sneezy")

    def test_17(self):
        "Ensure that transformation resolves page templates correctly"
        reg = TransformerRegistry.addInstance(self.folder,'registry')

        xsltFile     = open('testfiles/templatetest.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        self.folder.who = "sneezy"
        
        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        xform.behave_like = 'PageTemplate'
        result = xform(xform, self.app.REQUEST)
        self.failIf(result.find("Hello, sneezy") == -1)

    ################################################################
    # Test Runner Setup
    ################################################################

if __name__ == '__main__':
    framework(descriptions=1, verbosity=2)
else:
    # While framework.py provides its own test_suite() 
    # method the testrunner utility does not.
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(XMLTransformTestCase))
        return suite
